@@ -11,7 +11,7 @@ our @ISA = qw(Exporter);
 our @EXPORT_OK = qw($JCODE_COMPAT $VERSION);
 
 our @EXPORT = qw(mime_eco);
-our $VERSION = '0.50';
+our $VERSION = '0.60';
 
 use constant HEAD   => '=?UTF-8?B?';
 use constant HEAD_J => '=?ISO-2022-JP?B?';
@@ -24,17 +24,21 @@ sub mime_eco {
     my $bpl = shift || 76;
     my $mode = shift;
     $mode = 2 unless defined $mode;
+    my $lss = shift;
+    $lss = 25 unless defined $lss;
+
     my $csn;
-    my $pos = 0;
+    my $pos;
     my $np;
     my $refsub;
+    my $reg;
 
     my ($w1, $w1_len, $w2);
     my ($sps, $sps_len);
-    my $sp0;
     my $sp1 = '';
-    my $result = '';
-    my $ascii = 1;
+    my $sp1_bak;
+    my $result;
+    my $ascii;
     my $tmp;
     my $count = 0;
 
@@ -56,8 +60,27 @@ sub mime_eco {
 
     $str =~ /(\s*)(\S+)/gc;
     ($sps, $w2) = ($1, $2);
-    $ascii = 0 if $w2 =~ /[^\x21-\x7e]/;
-    $w1 = "$sps$w2";
+
+    if ($w2 =~ /[^21-\x7e]/) {
+	$ascii = 0;
+	$sps_len = length($sps);
+	if ($sps_len > $lss) {
+	    $result = substr($sps, 0, $lss);
+	    $w1 = substr($sps, $lss) . $w2;
+	    $pos = $lss;
+	}
+	else {
+	    $result = $sps;
+	    $w1 = $w2;
+	    $pos = $sps_len;
+	}
+    }
+    else {
+	$ascii = 1;
+	$result = '';
+	$w1 = "$sps$w2";
+	$pos = 0;
+    }
 
     if ($mode == 2) {
 	$mode = ($w1 =~ /^(?:Subject:|Comments:)$/i) ? 0 : 1;
@@ -73,11 +96,12 @@ sub mime_eco {
 
     while ($str =~ /(\s*)(\S+)/gc) {
 	($sps, $w2) = ($1, $2);
-
 	if ($w2 =~ /[^\x21-\x7e]/) {
+	    $sps_len = length($sps);
 	    if ($ascii) { # "ASCII \s+ WCHAR"
-		$sp0 = chop($sps);
-		$w1 .= $sps;
+		$sp1_bak = $sp1;
+		$sp1 = chop($sps);
+		$w1 .= $sps if $sps_len > $lss;
 		$w1_len = length($w1);
 		if ($count == 0) {
 		    $result = $w1;
@@ -85,19 +109,62 @@ sub mime_eco {
 		}
 		else {
 		    if (($count > 1) and ($pos + $w1_len + 1 > $bpl)) {
-			$result .= "$lf$sp1$w1";
-			$pos = $w1_len + 1;
+                        $result .= "$lf$sp1_bak$w1";
+                        $pos = $w1_len + 1;
+                    }
+                    else {
+                        $result .= "$sp1_bak$w1";
+                        $pos += $w1_len + 1;
+                    }
+		}
+		if ($sps_len <= $lss) {
+		    if ($pos + $sps_len - 1 > $bpl) {
+			$result .= substr($sps, 0, $bpl - $pos) . $lf
+			    . substr($sps, $bpl - $pos);
+			$pos += $sps_len - $bpl - 1;
 		    }
 		    else {
-			$result .= "$sp1$w1";
-			$pos += $w1_len + 1;
+			$result .= $sps;
+			$pos += $sps_len - 1;
 		    }
 		}
-		$sp1 = $sp0;
 		$w1 = $w2;
 	    }
 	    else { # "WCHAR \s+ WCHAR"
-		$w1 .= "$sps$w2";
+		if (($mode == 1) and ($sps_len <= $lss)) {
+		    $reg = ($csn == 1) ?
+			qr/\)\,?$/ : qr/\e\(B[\x21-\x7e]*\)\,?$/;
+		    if ($w1 =~ /$reg/ or $w2 =~ /^\(/) {
+			if ($count == 0) {
+			    $result .=
+				&$refsub($w1, $pos, $lf, $bpl, \$np, 0, $csn);
+			}
+			else {
+			    $tmp = &$refsub($w1, 1 + $pos,
+					    $lf, $bpl, \$np, 0, $csn);
+			    $result .= ($tmp =~ s/^ /$sp1/) ?
+				"$lf$tmp" : "$sp1$tmp";
+			}
+			$pos = $np;
+			$sp1 = chop($sps);
+			if ($pos + $sps_len - 1 > $bpl) {
+			    $result .= substr($sps, 0, $bpl - $pos) . $lf
+				. substr($sps, $bpl - $pos);
+			    $pos += $sps_len - $bpl - 1;
+			}
+			else {
+			    $result .= $sps;
+			    $pos += $sps_len - 1;
+			}
+			$w1 = $w2;
+		    }
+		    else {
+			$w1 .= "$sps$w2";
+		    }
+		}
+		else {
+		    $w1 .= "$sps$w2";
+		}
 	    }
 	    $ascii = 0;
 	}
@@ -121,35 +188,33 @@ sub mime_eco {
 	    }
 	    else { # "WCHAR \s+ ASCII"
 		if ($count == 0) {
-                    $result = &$refsub($w1, 0, $lf, $bpl, \$np, 0, $csn);
+		    $result .=
+			&$refsub($w1, $pos, $lf, $bpl, \$np, 0, $csn);
                     $pos = $np;
                 }
 		else {
 		    $tmp =
 			&$refsub($w1, 1 + $pos, $lf, $bpl, \$np, 0, $csn);
-
 		    $result .= ($tmp =~ s/^ /$sp1/) ? "$lf$tmp" : "$sp1$tmp";
 		    $pos = $np;
 		}
 	    }
 	    $sps_len = length($sps);
-
 	    if ($pos >= $bpl) {
-		$sp0 = substr($sps, 0, 1);
+		$sp1 = substr($sps, 0, 1);
 		$w2 = substr($sps, 1) . $w2;
 	    }
 	    elsif ($pos + $sps_len - 1 > $bpl) {
 		$result .= substr($sps, 0, $bpl - $pos);
-		$sp0 = substr($sps, $bpl - $pos, 1);
+		$sp1 = substr($sps, $bpl - $pos, 1);
 		$w2 = substr($sps, $bpl - $pos + 1) . $w2;
 		$pos = $bpl;
 	    }
 	    else {
-		$sp0 = chop($sps);
+		$sp1 = chop($sps);
 		$result .= $sps;
 		$pos += $sps_len - 1;
 	    }
-	    $sp1 = $sp0;
 	    $w1 = $w2;
 	    $ascii = 1;
 	}
@@ -157,8 +222,8 @@ sub mime_eco {
     }
     ($sps) = ($str =~ /(.*)/g); # All space of the remainder
 
-    $w1 .= $sps;
     if ($ascii) {
+	$w1 .= $sps;
 	if ($count == 0) {
 	    $result = $w1;
 	}
@@ -173,20 +238,39 @@ sub mime_eco {
 	}
     }
     else {
+	$sps_len = length($sps);
 	if ($count == 0) {
-	    $result = &$refsub($w1, 0, $lf, $bpl, \$np, 0, $csn);
+	    if ($sps_len > $lss) {
+		$w1 .= substr($sps, 0, $sps_len - $lss);
+		$result .=
+		    &$refsub($w1, $pos, $lf, $bpl, \$np, $lss, $csn)
+			. substr($sps, $sps_len - $lss);
+	    }
+	    else {
+		$result .=
+		    &$refsub($w1, $pos, $lf, $bpl, \$np, $sps_len, $csn) .
+                        $sps;
+	    }
 	}
 	else {
-	    $tmp = &$refsub($w1, 1 + $pos, $lf, $bpl, \$np, 0, $csn);
+	    if ($sps_len > $lss) {
+		$w1 .= substr($sps, 0, $sps_len - $lss);
+		$tmp = &$refsub($w1, 1 + $pos, $lf, $bpl, \$np, $lss, $csn)
+		    . substr($sps, $sps_len - $lss);
+	    }
+	    else {
+		$tmp =
+		    &$refsub($w1, 1 + $pos, $lf, $bpl, \$np, $sps_len, $csn)
+			. $sps;
+	    }
 	    $result .= ($tmp =~ s/^ /$sp1/) ? "$lf$tmp" : "$sp1$tmp";
 	}
     }
-
     return $trailing_crlf ? $result . $trailing_crlf : $result;
 }
 
 
-# add encorded-word (for structured header)
+# add encoded-word (for structured header)
 #   parameters:
 #     str : 7bit-jis string
 #     sp  : start position (indentation of the first line)
@@ -198,46 +282,46 @@ sub mime_eco {
 sub add_enc_word_sth {
     my($str, $sp, $lf, $bpl, $ep, $rll, $csn) = @_;
 
-    my ($rb1, $rb2) = (0, 0); # '(' and ')' : round brackets
-    my $rb2s;
+    my ($rb1, $rb2); # '(' and ')' : round brackets
+    my ($rb1_len, $rb2_len) = (0, 0);
     my $reg;
     my $refsub;
     my $tmp;
 
     if ($csn == 1) {
-	$reg = qr/(\)\,?)$/;
+	$reg = qr/(\){1,3}\,?)$/;
 	$refsub = \&add_enc_word_utf8;
     }
     else {
-	$reg = qr/\e\(B[\x21-\x7e]*(\)\,?)$/;
+	$reg = qr/\e\(B[\x21-\x7e]*?(\){1,3}\,?)$/;
 	$refsub = \&add_enc_word_7bit_jis;
     }
-
-    if ($str =~ s/^\(//) {
-	$rb1 = 1;
-	$sp += 1;
+    if ($str =~ s/^(\({1,3})//) {
+	$rb1 = $1;
+	$rb1_len = length($rb1);
+	$sp += $rb1_len;
     }
     if ($str =~ /$reg/) {
-	$rb2s = $1;
-	$rb2 = 1;
-	chop($str);
-	$rll = length($rb2s);
-	chop($str) if $rll > 1;
+	$rb2 = $1;
+	$rb2_len = length($rb2);
+	$rll = $rb2_len;
+	substr($str, -$rb2_len) = '';
     }
     $tmp = &$refsub($str, $sp, $lf, $bpl, $ep, $rll, $csn);
-
-    if ($rb1) {
-	if ($tmp !~ s/^ / \(/) {
-	    $tmp = '(' . $tmp;
+    if ($rb1_len > 0) {
+	if ($tmp !~ s/^ / $rb1/) {
+	    $tmp = $rb1 . $tmp;
 	}
     }
-    $tmp .= $rb2s if $rb2;
-
+    if ($rb2_len > 0) {
+	$tmp .= $rb2;
+	$$ep += $rb2_len;
+    }
     return $tmp;
 }
 
 
-# add encorded-word for 7bit-jis string
+# add encoded-word for 7bit-jis string
 sub add_enc_word_7bit_jis {
     require MIME::Base64;
     my($str, $sp, $lf, $bpl, $ep, $rll, $csn) = @_;
@@ -259,10 +343,12 @@ sub add_enc_word_7bit_jis {
 
     # encoded size + sp (18 is HEAD_J + TAIL)
     my $ep_tmp = int(($str_len + 2) / 3) * 4 + 18 + $sp;
+
     if ($ep_tmp + $rll <= $bpl) {
 	$$ep = $ep_tmp;
 	return HEAD_J . MIME::Base64::encode_base64($str, '') . TAIL;
     }
+    $ll_flag = 1 if $ep_tmp <= $bpl;
     while ($str =~ /\e(..)|(.)/g) {
 	($ec, $c) = ($1, $2);
 	if (defined $ec) {
@@ -326,15 +412,13 @@ sub add_enc_word_7bit_jis {
 		$chunk = $w . substr($str, $str_pos);
 		last;
 	    }
-	    elsif ($ep_tmp <= $bpl) {
-		$ll_flag = 1;
-	    }
+	    $ll_flag = 1 if $ep_tmp <= $bpl;
             $chunk = $w;
             $chunk_len = $w_len;
             $sp = 1; # 1 is top space
         }
         else {
-	    if ($ll_flag and pos($str) == $str_len) {
+	    if ($ll_flag and pos($str) == $str_len) { # last char
 		if ($k_in_bak) {
 		    $chunk .= "\e\(B";
 		    $w = "\e$ec_bak" . $w;
@@ -358,7 +442,7 @@ sub add_enc_word_7bit_jis {
 }
 
 
-# add encorded-word for utf8 string
+# add encoded-word for utf8 string
 sub add_enc_word_utf8 {
     require MIME::Base64;
     my($str, $sp, $lf, $bpl, $ep, $rll, $csn) = @_;
@@ -381,6 +465,16 @@ sub add_enc_word_utf8 {
 
     utf8::decode($str); # UTF8 flag on
 
+    if ($ep_tmp <= $bpl) {
+	my $w = chop($str);
+	utf8::encode($w); # UTF8 flag off
+	$$ep = int((length($w) + 2) / 3) * 4 + 13; # 13 is 12 + space
+	utf8::encode($str); # UTF8 flag off
+	$result = ($str eq '') ? ' ' :
+	    HEAD . MIME::Base64::encode_base64($str, '') . TAIL . "$lf ";
+	return $result . HEAD . MIME::Base64::encode_base64($w, '') . TAIL;
+    }
+
     for my $w (split //, $str) {
 	utf8::encode($w); # UTF8 flag off
 	$w_len = length($w); # size of one character
@@ -405,7 +499,7 @@ sub add_enc_word_utf8 {
                 $chunk = substr($str, $str_pos);
                 last;
             }
-	    elsif ($ep_tmp <= $bpl) {
+	    if ($ep_tmp <= $bpl) {
 		$w = chop($str);
 		utf8::encode($w); # UTF8 flag off
 		$w_len = length($w);
@@ -413,7 +507,7 @@ sub add_enc_word_utf8 {
 		$chunk = substr($str, $str_pos);
 		$result .= HEAD .
 		    MIME::Base64::encode_base64($chunk, '') . TAIL . "$lf ";
-		$ep_tmp = int(($w_len + 2) / 3) * 4 + 13;
+		$ep_tmp = int(($w_len + 2) / 3) * 4 + 13; # 13 is 12 + space
 		$chunk = $w;
 		last;
 	    }
@@ -449,7 +543,7 @@ This module implements RFC 2047 Mime Header Encoding.
 
 =head2 OPTIONS
 
-  $encoded = mime_eco($str, $charset, $lf, $bpl, $mode);
+  $encoded = mime_eco($str, $charset, $lf, $bpl, $mode, $lss);
                # $charset : 'UTF-8' or 'ISO-2022-JP'
                # $lf      : line feed (default: "\n")
                # $bpl     : bytes per line (default: 76)
@@ -457,6 +551,7 @@ This module implements RFC 2047 Mime Header Encoding.
                #            1 : structured header
                #            2 : auto (Subject or Comments ? 0 : 1)
                #            (default: 2)
+               # $lss     : length of security space (default: 25)
 
 =head1 SEE ALSO
 
@@ -468,7 +563,7 @@ MURATA Yasuhisa E<lt>murata@nips.ac.jpE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2011 MURATA Yasuhisa
+Copyright (C) 2011-2012 MURATA Yasuhisa
 
 =head1 LICENSE
 
